import fs from 'fs';
import CONTENT_TYPE from './mime/mime';

/**
 * 判断url是否符合concat请求规则
 * @param url
 * @returns {boolean}
 * @private
 */
function _checkIsNeedConcat(url) {
    let isNeedConcat = false;
    url.indexOf('??') > 0 ? isNeedConcat = true : isNeedConcat = false;
    return isNeedConcat;
}

class Concat {
    constructor(options) {
        this.options = options;
        this.init = this.init.bind(this);
    }

    /**
     * 用buffer拼接文件
     * @param url
     * @returns {Promise.<{bf: (Array.<T>|string), contentType: *}>}
     */
    async concatFile(url) {
        const urlSplit = url.split('??');
        const urlPrefix = urlSplit[0];
        const urlFiles = urlSplit[1].split('?')[0];
        const files = urlFiles.split(',');
        const staticPath = this.options.staticPath;
        const fileBufferArray = [];
        let contentType = files[0].split('.')[1];

        for (let i = 0, len = files.length; i < len; i++) {
            const fileI = await fs.readFileSync(staticPath + urlPrefix + files[i]);
            if (i > 0 && i < len) {
                let enter = new Buffer.from([0x0a]);
                fileBufferArray.push(enter);
            }
            fileBufferArray.push(fileI);
        }
        return {bf: Buffer.concat(fileBufferArray), contentType: contentType};
    }

    /**
     * concat中间件
     * @param ctx
     * @param next
     * @returns {Promise.<void>}
     */
    async init(ctx, next) {
        const isNeedConcat = _checkIsNeedConcat(ctx.req.url);
        if (isNeedConcat) {
            const concatResult = await this.concatFile(ctx.req.url);
            ctx.response.set('Content-Type', CONTENT_TYPE[concatResult.contentType]);
            ctx.body = concatResult.bf.toString('utf8');
        } else {
            await next();
        }
    }

}

export default Concat;


